@@ -71,9 +71,11 @@ struct _GLogHandler
 
 
 /* --- prototypes --- */
+#ifndef HAVE_C99_VSNPRINTF
 static gsize printf_string_upper_bound (const gchar *format,
 					gboolean     may_warn,
 					va_list      args);
+#endif /* !HAVE_C99_VSNPRINTF */
 
 
 /* --- variables --- */
@@ -477,7 +479,7 @@ g_logv (const gchar   *log_domain,
 
 #ifndef  HAVE_VSNPRINTF
   va_list args2;
-#endif	/* !HAVE_VSNPRINTF */
+#endif	/* HAVE_VSNPRINTF */
   
   log_level &= G_LOG_LEVEL_MASK;
   if (!log_level)
@@ -535,21 +537,17 @@ g_logv (const gchar   *log_domain,
 	  
 	  if (test_level & G_LOG_FLAG_FATAL)
 	    {
-#if defined (G_ENABLE_DEBUG) && defined (SIGTRAP)
+#ifdef G_OS_WIN32
+	      MessageBox (NULL, fatal_msg_buf, NULL, MB_OK);
+#endif
+#if defined (G_ENABLE_DEBUG) && (defined (SIGTRAP) || defined (G_OS_WIN32))
 	      if (!(test_level & G_LOG_FLAG_RECURSION))
 		G_BREAKPOINT ();
 	      else
 		abort ();
-#else /* !G_ENABLE_DEBUG || !SIGTRAP */
-#ifdef G_OS_WIN32
-	      MessageBox (NULL, fatal_msg_buf, NULL, MB_OK);
-#endif
-# if defined (_MSC_VER) && defined (_DEBUG)
-	      /* let's see the call stack ... */
-	      __asm int 3
-# endif
+#else /* !G_ENABLE_DEBUG || !(SIGTRAP || G_OS_WIN32) */
 	      abort ();
-#endif /* !G_ENABLE_DEBUG || !SIGTRAP */
+#endif /* !G_ENABLE_DEBUG || !(SIGTRAP || G_OS_WIN32) */
 	    }
 	  
 	  depth--;
@@ -860,6 +858,8 @@ g_printerr (const gchar *format,
 #  define MB_LEN_MAX 8
 #endif
 
+#ifndef HAVE_C99_VSNPRINTF
+
 typedef struct
 {
   guint min_width;
@@ -1168,11 +1168,19 @@ printf_string_upper_bound (const gchar *format,
   return len;
 }
 
+#endif /* !HAVE_C99_VSNPRINTF */
+
+
 gsize
 g_printf_string_upper_bound (const gchar *format,
 			     va_list      args)
 {
+#if HAVE_C99_VSNPRINTF
+  gchar c;
+  return vsnprintf (&c, 1, format, args) + 1;
+#else
   return printf_string_upper_bound (format, TRUE, args);
+#endif
 }
 
 void
